use grammers_client::{Client, Update};
use rand::seq::SliceRandom;

use crate::{bot::handlers::Handler, utils::messages::get_message};

#[derive(Clone)]
pub struct Chooser;

#[async_trait::async_trait]
impl Handler for Chooser {
    async fn react(&self, _: &Client, update: &Update) -> anyhow::Result<()> {
        let Some(input_message) = get_message(update) else {
            return Ok(());
        };
        let response = input_message
            .text()
            .strip_prefix(".c")
            // Check if string is not empty.
            .filter(|a| !a.is_empty())
            // Split by commas.
            .map(|text| text.trim().split(',').collect::<Vec<_>>())
            // Choose random variant.
            .and_then(|collected| collected.choose(&mut rand::rngs::OsRng).copied());
        // It the string is chosen, reply to message with it.
        if let Some(answer) = response {
            input_message.reply(answer).await?;
        } else {
            input_message
                .reply("Я не смог понять из чего мне выбирать. Попробуй ещё раз.")
                .await?;
        }
        Ok(())
    }
}
