use grammers_client::{Client, InputMessage, Update};
use rand::seq::SliceRandom;

use crate::{bot::handlers::Handler, utils::messages::get_message};

lazy_static::lazy_static! {
    static ref ANSWERS: &'static [&'static str] = &[
        // Positives
        "Это точно.",
        "Решительно так.",
        "Без тени сомнения.",
        "Определённо, да.",
        "Можешь расчитывать на милость своих богов.",
        "Да, как я вижу.",
        "Скорее всего.",
        "Да",
        "Сегодня боги благосклонны к тебе. Да.",
        "Руны говорят мне \"да\"",
        // Non-committal
        "Боги скрыли от меня ответ, придётся решать самому.",
        "Я не могу прочитать нить твоей судьбы. Выбор за тобой.",
        "Сейчас тебе лучше не знать ответа.",
        "Пока что я не могу предсказать это.",
        "Сконцентрируйся и спроси ещё раз. Тогда боги точно услышат тебя",
        // Negatives
        "Не расчитывай на это",
        "Наш ответ - нет!",
        "Боги говорят нет.",
        "Не стоит.",
        "Определённо нет."
    ];
}

#[derive(Clone)]
pub struct MagicBall;

#[async_trait::async_trait]
impl Handler for MagicBall {
    async fn react(&self, _: &Client, update: &Update) -> anyhow::Result<()> {
        let Some(message) = get_message(update) else {
            return Ok(());
        };
        // If empty message passed.
        if message.text().strip_prefix(".mb").map_or("", str::trim) == "" {
            message
                .reply(InputMessage::text(
                    "Нужно спросить вопрос. Пустые строки не канают.",
                ))
                .await?;
            return Ok(());
        }
        message
            .reply(
                InputMessage::text(ANSWERS.choose(&mut rand::rngs::OsRng).copied().unwrap())
                    .silent(true),
            )
            .await?;
        Ok(())
    }
}
