use grammers_client::{Client, Update};
use grammers_tl_types::types::{InputDialogPeer, InputPeerChat};

use crate::{
    bot::{handlers::Handler, middlewares::base::Middleware},
    utils::messages::get_message,
};

#[derive(Clone)]
pub struct MarkUnread {
    handler: Box<dyn Handler>,
}

#[async_trait::async_trait]
impl Handler for MarkUnread {
    async fn react(&self, client: &Client, update: &Update) -> anyhow::Result<()> {
        let res = self.handler.react(client, update).await;

        if let Some(message) = get_message(update) {
            let res = client
                .invoke(&grammers_tl_types::functions::messages::MarkDialogUnread {
                    peer: grammers_tl_types::enums::InputDialogPeer::Peer(InputDialogPeer {
                        peer: grammers_tl_types::enums::InputPeer::Chat(InputPeerChat {
                            chat_id: message.chat().id(),
                        }),
                    }),
                    unread: true,
                })
                .await;
            if let Err(err) = res {
                log::warn!("Cannot mark dialog unread. Reason: {err}.");
            }
        }

        res
    }
}

impl Middleware for MarkUnread {
    fn from_handler(handler: Box<dyn Handler>) -> Self {
        Self { handler }
    }
}
