use grammers_client::{Client, Update};

use crate::{
    bot::{handlers::Handler, middlewares::base::Middleware},
    utils::messages::get_message,
};

#[derive(Clone)]
pub struct MembersCount<const MAX_COUNT: usize> {
    handler: Box<dyn Handler>,
}

#[async_trait::async_trait]
impl<const MAX_COUNT: usize> Handler for MembersCount<MAX_COUNT> {
    async fn react(&self, client: &Client, update: &Update) -> anyhow::Result<()> {
        if let Some(message) = get_message(update) {
            let participants = client
                .iter_participants(message.chat())
                .total()
                .await
                .unwrap_or(0);

            if participants > MAX_COUNT {
                log::warn!("Too many participants. Skipping.");
                return Ok(());
            }
        }
        self.handler.react(client, update).await
    }
}

impl<const MAX_COUNT: usize> Middleware for MembersCount<MAX_COUNT> {
    fn from_handler(handler: Box<dyn Handler>) -> Self {
        Self { handler }
    }
}
