use std::sync::Arc;

use actix_web::{
    get, post,
    web::{Data, Json},
    HttpResponse, Responder,
};
use tokio::sync::RwLock;

use super::{schema::LoginRequestDTO, templates::Index};
use crate::args::ServerConfig;

#[allow(clippy::unused_async)]
#[get("/health")]
async fn healthcheck() -> impl Responder {
    HttpResponse::Ok().finish()
}

#[post("/login")]
pub async fn login(
    code_data: Json<LoginRequestDTO>,
    token: Data<Arc<RwLock<Option<String>>>>,
    config: Data<ServerConfig>,
) -> impl Responder {
    if code_data.password != config.server_pass {
        log::warn!("Passwords don't match");
        return HttpResponse::BadRequest().body("Passwords don't match");
    }
    if token.read().await.is_some() {
        log::warn!("Token is already set.");
        return HttpResponse::Conflict().body("Token is already set");
    }
    let mut token_write = token.write().await;
    *token_write = Some(code_data.code.clone());
    log::info!("Token is updated!");
    HttpResponse::Ok().finish()
}

#[get("/")]
pub async fn index(
    token: Data<Arc<RwLock<Option<String>>>>,
    config: Data<ServerConfig>,
) -> impl Responder {
    Index {
        activated: token.read().await.is_some(),
        username: config.username.clone(),
        image_num: rand::random::<u8>() % 3,
    }
}
